import io
import pytest

import libconf


# Helper functions
##################

def dump_value(key, value, **kwargs):
    str_file = io.StringIO()
    libconf.dump_value(key, value, str_file, **kwargs)
    return str_file.getvalue()

def dump_collection(c, **kwargs):
    str_file = io.StringIO()
    libconf.dump_collection(c, str_file, **kwargs)
    return str_file.getvalue()

def dump_dict(c, **kwargs):
    str_file = io.StringIO()
    libconf.dump_dict(c, str_file, **kwargs)
    return str_file.getvalue()


# Actual tests
##############

def test_dump_collection():
    c = [1, 2, 3]
    c_dumped = dump_collection(c)
    assert c_dumped == '1,\n2,\n3'
    c = (1, 2, 3)
    c_dumped = dump_collection(c)
    assert c_dumped == '1,\n2,\n3'
    c = ((1, 2), (3))
    c_dumped = dump_collection(c)
    assert c_dumped == '(\n    1,\n    2\n),\n3'
    c = ([1, 2], (3))
    c_dumped = dump_collection(c)
    assert c_dumped == '[\n    1,\n    2\n],\n3'
    c = [[1, 2], (3)]
    c_dumped = dump_collection(c)
    assert c_dumped == '[\n    1,\n    2\n],\n3'
    c_dumped = dump_collection(c, indent=4)
    assert c_dumped == '    [\n        1,\n        2\n    ],\n    3'

def test_dump_dict_simple_string():
    c = {'name': 'value'}
    c_dumped = dump_dict(c)
    assert c_dumped == 'name = "value";\n'

def test_dump_dict_indentation_dicts():
    c = {'a': {'b': 3}}
    c_dumped = dump_dict(c)
    assert c_dumped == 'a =\n{\n    b = 3;\n};\n'

def test_dump_dict_indentation_dicts_with_extra_indent():
    c = {'a': {'b': 3}}
    c_dumped = dump_dict(c, indent=4)
    assert c_dumped == '    a =\n    {\n        b = 3;\n    };\n'

def test_dump_dict_indentation_dicts_within_lists():
    c = {'a': ({'b': 3},)}
    c_dumped = dump_dict(c)
    assert c_dumped == 'a =\n(\n    {\n        b = 3;\n    }\n);\n'

def test_dump_string_escapes_backslashes():
    s = r'abc \ def \ hij'
    c_dumped = libconf.dump_string(s)
    assert c_dumped == r'"abc \\ def \\ hij"'

def test_dump_string_escapes_doublequotes():
    s = r'abc "" def'
    c_dumped = libconf.dump_string(s)
    assert c_dumped == r'"abc \"\" def"'

def test_dump_string_escapes_common_escape_characters():
    s = '\f \n \r \t'
    c_dumped = libconf.dump_string(s)
    assert c_dumped == r'"\f \n \r \t"'

def test_dump_string_escapes_unprintable_characters():
    s = '\x00 \x1f \x7f'
    c_dumped = libconf.dump_string(s)
    assert c_dumped == r'"\x00 \x1f \x7f"'

def test_dump_string_keeps_8bit_chars_intact():
    s = '\x80 \x9d \xff'
    c_dumped = libconf.dump_string(s)
    assert c_dumped == '"\x80 \x9d \xff"'

def test_dump_string_handles_unicode_strings():
    s = u'\u2603'
    c_dumped = libconf.dump_string(s)
    assert c_dumped == u'"\u2603"'

def test_dump_boolean():
    c = (True, False)
    c_dumped = dump_collection(c)
    assert c_dumped == 'True,\nFalse'

def test_dump_int():
    assert libconf.dump_int(0) == '0'
    assert libconf.dump_int(-30) == '-30'
    assert libconf.dump_int(60) == '60'

def test_dump_int32_has_no_l_suffix():
    assert libconf.dump_int(2**31 - 1) == str(2**31 - 1)
    assert libconf.dump_int(-2**31) == str(-2**31)

def test_dump_int64_has_l_suffix():
    assert libconf.dump_int(2**31) == str(2**31) + 'L'
    assert libconf.dump_int(-2**31 - 1) == str(-2**31 - 1) + 'L'

def test_dump_raises_on_string_input():
    with pytest.raises(libconf.ConfigSerializeError):
        libconf.dumps("")

def test_dump_raises_on_list_input():
    with pytest.raises(libconf.ConfigSerializeError):
        libconf.dumps([])

def test_none_dict_value_raises():
    with pytest.raises(libconf.ConfigSerializeError):
        libconf.dumps({'test': None})

def test_none_dict_key_raises():
    with pytest.raises(libconf.ConfigSerializeError):
        libconf.dumps({None: 0})

def test_integer_dict_key_raises():
    with pytest.raises(libconf.ConfigSerializeError):
        libconf.dumps({0: 0})

def test_invalid_value_raises():
    with pytest.raises(libconf.ConfigSerializeError):
        libconf.dumps({'a': object()})

def test_array_with_composite_type_dict_raises():
    with pytest.raises(libconf.ConfigSerializeError):
        libconf.dumps({'a': [{}]})

def test_array_with_composite_type_list_raises():
    with pytest.raises(libconf.ConfigSerializeError):
        libconf.dumps({'a': [()]})

def test_array_with_composite_type_array_raises():
    with pytest.raises(libconf.ConfigSerializeError):
        libconf.dumps({'a': [[]]})

def test_array_with_mixed_types_intstr_raises():
    with pytest.raises(libconf.ConfigSerializeError):
        libconf.dumps({'a': [1, "str"]})

def test_array_with_mixed_types_intbool_raises():
    with pytest.raises(libconf.ConfigSerializeError):
        libconf.dumps({'a': [1, True]})

def test_array_with_intint64():
    c = {'a': [2, 2**65]}
    c_dumped = dump_dict(c).replace(" ", "").replace("\n", "")
    assert c_dumped == 'a=[2L,36893488147419103232L];'

def test_LibconfArray_produces_array():
    c = {'a': libconf.LibconfArray([1,2])}
    c_dumped = dump_dict(c).replace(" ", "").replace("\n", "")
    assert c_dumped == 'a=[1,2];'

def test_LibconfList_produces_list():
    c = {'a': libconf.LibconfList([1,2])}
    c_dumped = dump_dict(c).replace(" ", "").replace("\n", "")
    assert c_dumped == 'a=(1,2);'

def test_LibconfInt64_produces_L_suffix():
    c = {'a': libconf.LibconfInt64(2)}
    c_dumped = dump_dict(c).replace(" ", "").replace("\n", "")
    assert c_dumped == 'a=2L;'



import os
import io
import textwrap
import pytest

import libconf


CURDIR = os.path.abspath(os.path.dirname(__file__))


# Tests for load() and loads()
##############################

def test_loads_maintains_dict_order():
    config = libconf.loads(u'''l: 1; i: 5; b: 3; c: 1; o: 9; n: 0; f: 7;''')
    assert ''.join(config.keys()) == 'libconf'

def test_example_config():
    example_file = os.path.join(CURDIR, 'test_e2e.cfg')
    with io.open(example_file, 'r', encoding='utf-8') as f:
        c = libconf.load(f, includedir=CURDIR)

    assert c.appconfig.version == 37
    assert c.appconfig['version-long'] == 370000000000000
    assert c.appconfig['version-autolong'] == 370000000000000
    assert c.appconfig.name == "libconf"
    assert c.appconfig.delimiter == False
    assert c.appconfig.works == True
    assert c.appconfig.allows == 0xA
    assert c.appconfig['eol-comments'] == 0xA
    assert c.appconfig.list == (3, "chicken", (), dict(group=True))
    assert c.appconfig.sub_group.sub_sub_group.yes == "yes"
    assert c.appconfig.sub_group.sub_sub_group['include-works'] == True
    assert c.appconfig.sub_group.arr == [1, 2]
    assert c.appconfig.sub_group.str == "Strings are joined despite comments";

def test_string_merging():
    # Unicode characters are supported, \u escapes not.
    input = u"""s = "abc\x21def\n" /* comment */ "newline-" # second comment
                   "here \u2603 \\u2603";"""
    assert libconf.loads(input).s == u"abc\x21def\nnewline-here \u2603 \\u2603"

def test_nonexisting_include_raises():
    input = u'''@include "/NON_EXISTING_FILE/DOESNT_EXIST"'''
    with pytest.raises(libconf.ConfigParseError):
        libconf.loads(input)

def test_circular_include_raises():
    circular_file = os.path.join(CURDIR, 'circular1.cfg')
    with io.open(circular_file, 'r', encoding='utf-8') as f:
        with pytest.raises(libconf.ConfigParseError):
            libconf.load(f, includedir=CURDIR)

def test_loads_of_bytes_throws():
    with pytest.raises(TypeError) as excinfo:
        libconf.loads(b'')

    assert 'libconf.loads' in str(excinfo.value)

def test_load_of_BytesIO_throws():
    with pytest.raises(TypeError) as excinfo:
        libconf.load(io.BytesIO(b'a: "37";'))

    assert 'libconf.load' in str(excinfo.value)

def test_lists_support_trailing_comma():
    config = libconf.loads(u'''a: (1, 2, 3,);''')
    assert config.a == (1, 2, 3)

def test_arrays_support_trailing_comma():
    config = libconf.loads(u'''a: [1, 2, 3,];''')
    assert config.a == [1, 2, 3]

# Tests for dump() and dumps()
##############################

def test_dump_special_characters():
    d = {'a': ({'b': [u"\x00 \n \x7f abc \xff \u2603"]},)}
    s = libconf.dumps(d)

    expected = textwrap.dedent(u'''\
        a =
        (
            {
                b =
                [
                    "\\x00 \\n \\x7f abc \xff \u2603"
                ];
            }
        );
        ''')
    assert s == expected


# Tests for dump-load round trips
#################################

def test_dumps_roundtrip():
    example_file = os.path.join(CURDIR, 'test_e2e.cfg')
    with io.open(example_file, 'r', encoding='utf-8') as f:
        c = libconf.load(f, includedir=CURDIR)

    c_dumped = libconf.loads(libconf.dumps(c))

    assert c == c_dumped

def test_dump_roundtrip():
    example_file = os.path.join(CURDIR, 'test_e2e.cfg')
    with io.open(example_file, 'r', encoding='utf-8') as f:
        c = libconf.load(f, includedir=CURDIR)

    with io.StringIO() as f:
        libconf.dump(c, f)
        f.seek(0)
        c_dumped = libconf.load(f, includedir=CURDIR)

    assert c == c_dumped

def test_dump_special_characters_roundtrip():
    d = {'a': ({'b': [u"\x00 \n \x7f abc \xff \u2603"]},)}
    d2 = libconf.loads(libconf.dumps(d))
    assert d == d2

def test_roundtrip_preserves_config_entry_order():
    config = libconf.loads(u'''l: 1; i: 5; b: 3; c: 1; o: 9; n: 0; f: 7;''')
    dumped = libconf.dumps(config)
    reloaded = libconf.loads(dumped)

    assert ''.join(reloaded.keys()) == 'libconf'

def test_roundtrip_of_int64_values():
    s = u'a=2L;'
    s2 = libconf.dumps(libconf.loads(s))
    assert s == s2.replace(' ', '').replace('\n', '')



from setuptools import setup

setup(
    name='libconf',
    version='2.0.1',
    description="A pure-Python libconfig reader/writer with permissive license",
    long_description=open('README.rst').read(),
    author="Christian Aichinger",
    author_email="Greek0@gmx.net",
    url='https://github.com/Grk0/python-libconf',
    download_url='https://github.com/Grk0/python-libconf/tarball/2.0.1',
    license="MIT",
    py_modules=['libconf'],
    keywords='libconfig configuration parser library',
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
    ],
)



import pytest

import libconf


def test_float():
    tokenizer = libconf.Tokenizer("<memory>")

    tokens = list(tokenizer.tokenize(" 2.  .5  0.75  1.0E1 "
                                     "+2. +.5 +0.75 +1.0E1 "
                                     "-2. -.5 -0.75 -1.0E1 "
                                     "2.E3 .5E6 0.75E9 1.0E1 "
                                     "2.E+3 .5E+6 0.75E+9 1.0E+1 "
                                     "2.E-3 .5E-6 0.75E-9 1.0E-1 "
                                     "2E1 -2e1 +2e1 5e-1 "
                                    ))

    assert [t.type for t in tokens] == ['float'] * 28
    assert [t.value for t in tokens][0:4] == [2.0, 0.5, 0.75, 10.0]
    assert [t.value for t in tokens][4:8] == [2.0, 0.5, 0.75, 10.0]
    assert [t.value for t in tokens][8:12] == [-2.0, -0.5, -0.75, -10.0]
    assert [t.value for t in tokens][12:16] == [2E3, .5E6, .75E9, 10]
    assert [t.value for t in tokens][16:20] == [2E3, .5E6, .75E9, 10]
    assert [t.value for t in tokens][20:24] == [2E-3, .5E-6, .75E-9, 0.1]
    assert [t.value for t in tokens][24:28] == [20.0, -20.0, 20.0, 0.5]

def test_hex64():
    tokenizer = libconf.Tokenizer("<memory>")

    tokens = list(tokenizer.tokenize("0x13AL 0XbcdL 0xefLL 0X456789ABLL"))

    assert [t.type for t in tokens] == ['hex64'] * 4
    assert [t.value for t in tokens] == [0x13A, 0xBCD, 0xef, 0x456789AB]

def test_hex():
    tokenizer = libconf.Tokenizer("<memory>")

    tokens = list(tokenizer.tokenize("0x13A 0Xbcd 0xef 0X456789AB"))

    assert [t.type for t in tokens] == ['hex'] * 4
    assert [t.value for t in tokens] == [0x13A, 0xBCD, 0xef, 0x456789AB]

def test_integer64():
    tokenizer = libconf.Tokenizer("<memory>")

    tokens = list(tokenizer.tokenize("10L +30L -15000000000LL"))

    assert [t.type for t in tokens] == ['integer64'] * 3
    assert [t.value for t in tokens] == [10, 30, -15000000000]

def test_integer():
    tokenizer = libconf.Tokenizer("<memory>")

    tokens = list(tokenizer.tokenize("10 +30 -15000000000"))

    assert [t.type for t in tokens] == ['integer'] * 3
    assert [t.value for t in tokens] == [10, 30, -15000000000]

def test_boolean():
    tokenizer = libconf.Tokenizer("<memory>")

    tokens = list(tokenizer.tokenize("true TRUE TrUe false FALSE FaLsE"))

    assert [t.type for t in tokens] == ['boolean'] * 6
    assert [t.value for t in tokens] == [True, True, True, False, False, False]

def test_string():
    tokenizer = libconf.Tokenizer("<memory>")

    tokens = list(tokenizer.tokenize(
        r'''"abc" "ab\"cd" "" "\x20\\\f\n\r\t"'''))

    assert [t.type for t in tokens] == ['string'] * 4
    assert [t.value for t in tokens] == ['abc', 'ab"cd', '', ' \\\f\n\r\t']

def test_name():
    tokenizer = libconf.Tokenizer("<memory>")

    tokens = list(tokenizer.tokenize("ident IdenT I I32A true_value falseVal"))

    assert [t.type for t in tokens] == ['name'] * 6
    assert [t.text for t in tokens] == ['ident', 'IdenT', 'I', 'I32A',
                                        'true_value', 'falseVal']

def test_special():
    tokenizer = libconf.Tokenizer("<memory>")

    tokens = list(tokenizer.tokenize("}]){[(=:,;"))

    assert [t.type for t in tokens] == list("}]){[(=:,;")

def test_location():
    tokenizer = libconf.Tokenizer("<memory>")

    tokens = list(tokenizer.tokenize("\n    0   1\n        2"))

    assert [t.type for t in tokens] == ['integer'] * 3
    assert [(t.row, t.column) for t in tokens] == [(2, 5), (2, 9), (3, 9)]

def test_invalid_token():
    tokenizer = libconf.Tokenizer("<memory>")

    with pytest.raises(libconf.ConfigParseError) as exc_info:
        list(tokenizer.tokenize("\n\n        `xvz"))

    assert '<memory>' in str(exc_info.value)
    assert 'row 3' in str(exc_info.value)
    assert 'column 9' in str(exc_info.value)
    assert '`xvz' in str(exc_info.value)

